@@ -20,27 +20,26 @@ namespace Sass {
 
     // statements
     virtual void operator()(Block*);
-    virtual void operator()(Ruleset*);
+    virtual void operator()(StyleRule*);
     virtual void operator()(Bubble*);
-    virtual void operator()(Supports_Block*);
-    virtual void operator()(Media_Block*);
-    virtual void operator()(At_Root_Block*);
-    virtual void operator()(Directive*);
+    virtual void operator()(SupportsRule*);
+    virtual void operator()(AtRootRule*);
+    virtual void operator()(AtRule*);
     virtual void operator()(Keyframe_Rule*);
     virtual void operator()(Declaration*);
     virtual void operator()(Assignment*);
     virtual void operator()(Import*);
     virtual void operator()(Import_Stub*);
-    virtual void operator()(Warning*);
-    virtual void operator()(Error*);
-    virtual void operator()(Debug*);
+    virtual void operator()(WarningRule*);
+    virtual void operator()(ErrorRule*);
+    virtual void operator()(DebugRule*);
     virtual void operator()(Comment*);
     virtual void operator()(If*);
-    virtual void operator()(For*);
-    virtual void operator()(Each*);
-    virtual void operator()(While*);
+    virtual void operator()(ForRule*);
+    virtual void operator()(EachRule*);
+    virtual void operator()(WhileRule*);
     virtual void operator()(Return*);
-    virtual void operator()(Extension*);
+    virtual void operator()(ExtendRule*);
     virtual void operator()(Definition*);
     virtual void operator()(Mixin_Call*);
     virtual void operator()(Content*);
@@ -63,15 +62,18 @@ namespace Sass {
     virtual void operator()(String_Quoted*);
     virtual void operator()(Custom_Error*);
     virtual void operator()(Custom_Warning*);
-    virtual void operator()(Supports_Operator*);
-    virtual void operator()(Supports_Negation*);
-    virtual void operator()(Supports_Declaration*);
+    virtual void operator()(SupportsOperation*);
+    virtual void operator()(SupportsNegation*);
+    virtual void operator()(SupportsDeclaration*);
     virtual void operator()(Supports_Interpolation*);
+    virtual void operator()(MediaRule*);
+    virtual void operator()(CssMediaRule*);
+    virtual void operator()(CssMediaQuery*);
     virtual void operator()(Media_Query*);
     virtual void operator()(Media_Query_Expression*);
     virtual void operator()(At_Root_Query*);
     virtual void operator()(Null*);
-    virtual void operator()(Parent_Selector* p);
+    virtual void operator()(Parent_Reference* p);
     // parameters and arguments
     virtual void operator()(Parameter*);
     virtual void operator()(Parameters*);
@@ -79,19 +81,19 @@ namespace Sass {
     virtual void operator()(Arguments*);
     // selectors
     virtual void operator()(Selector_Schema*);
-    virtual void operator()(Placeholder_Selector*);
-    virtual void operator()(Type_Selector*);
-    virtual void operator()(Class_Selector*);
-    virtual void operator()(Id_Selector*);
-    virtual void operator()(Attribute_Selector*);
-    virtual void operator()(Pseudo_Selector*);
-    virtual void operator()(Wrapped_Selector*);
-    virtual void operator()(Compound_Selector*);
-    virtual void operator()(Complex_Selector*);
-    virtual void operator()(Selector_List*);
-
-    virtual std::string lbracket(List*);
-    virtual std::string rbracket(List*);
+    virtual void operator()(PlaceholderSelector*);
+    virtual void operator()(TypeSelector*);
+    virtual void operator()(ClassSelector*);
+    virtual void operator()(IDSelector*);
+    virtual void operator()(AttributeSelector*);
+    virtual void operator()(PseudoSelector*);
+    virtual void operator()(SelectorComponent*);
+    virtual void operator()(SelectorCombinator*);
+    virtual void operator()(CompoundSelector*);
+    virtual void operator()(ComplexSelector*);
+    virtual void operator()(SelectorList*);
+    virtual sass::string lbracket(List*);
+    virtual sass::string rbracket(List*);
 
   };
 
