@@ -3,6 +3,8 @@ require 'set'
 require 'yaml'
 require 'command_line_reporter'
 
+require_relative 'directory'
+
 class SassSpec::Annotate::CLI
   include CommandLineReporter
 
@@ -52,18 +54,6 @@ BANNER
         options[:end_version] = version
       end
 
-      opts.on("--output-style STYLE",
-              [:expanded, :compact, :nested, :compressed, :unset],
-              "Set the output style for the specified tests.") do |output_style|
-        output_style = nil if output_style =~ /unset/i
-        options[:output_style] = output_style
-      end
-
-      opts.on("--[no-]clean-output", "Do some basic whitespace normalization.") do |clean|
-        clean = nil unless clean
-        options[:clean] = clean
-      end
-
       opts.on("--precision INTEGER",
               "Set the numeric output precision for the specified tests.",
               "Pass a precision of 'unset' to remove the precision.") do |precision|
@@ -210,7 +200,7 @@ BANNER
 
   def report_path(path)
     test_case_dirs = Dir.glob(File.join(path, "**/input.scss")).map {|p| File.dirname(p) }.uniq.sort
-    metadatas = test_case_dirs.map{|d| SassSpec::TestCaseMetadata.new(d) }
+    metadatas = test_case_dirs.map {|d| SassSpec::TestCaseMetadata.new(SassSpec::Directory.new(d))}
     TermInfo.screen_size
     max_length = [
       metadatas.map {|m| m.name.length}.max,
