@@ -22,6 +22,7 @@ end
 
 class ExecutableEngineAdapter < EngineAdapter
   include CaptureWithTimeout
+  attr_accessor :args
 
   def initialize(command, description = nil)
     @command = command
@@ -42,7 +43,8 @@ class ExecutableEngineAdapter < EngineAdapter
     command = File.absolute_path(@command)
     dirname, basename = File.split(sass_filename)
     result = capture3_with_timeout(
-      command, "--precision", precision.to_s, "-t", "expanded", basename,
+      command, "--precision", precision.to_s, "-t", "expanded",
+      "-I", File.expand_path("../../../spec", __FILE__), *@args&.split(/\s+/), basename,
       binmode: true, timeout: @timeout, chdir: dirname)
 
     if result[:timeout]
@@ -93,7 +95,7 @@ class DartEngineAdapter < EngineAdapter
           }
         }
       DART
-      @stdin, @stdout, @stderr = Open3.popen3("dart --packages=#{@path}/.packages #{f.path}")
+      @stdin, @stdout, @stderr = Open3.popen3("dart --enable-asserts --packages=#{@path}/.packages #{f.path}")
       @stdout.set_encoding 'binary'
       @stderr.set_encoding 'binary'
     end
@@ -107,7 +109,8 @@ class DartEngineAdapter < EngineAdapter
   def compile(sass_filename, precision)
     dirname, basename = File.split(sass_filename)
     @stdin.puts "!cd #{File.absolute_path(dirname)}"
-    @stdin.puts "--no-color --no-unicode #{@args} #{basename}"
+    @stdin.puts("--no-color --no-unicode -I #{File.expand_path("../../../spec", __FILE__)} " +
+                "#{@args} #{basename}")
     [next_chunk(@stdout), next_chunk(@stderr), next_chunk(@stdout).to_i]
   end
 
